import * as Config from "./modules/Config";
import * as Server from "./modules/Server";
import Log from "./modules/Log";
import UserProperty from "./modules/UserProperty";
import * as LocalData from "./modules/LocalData";

let g_instance = null;
export default class App {
    constructor() {
        this._autoPageView = true;
        this._actionQueue = [];
    }

    start() {
        this._processFunctionCall();

        if (!Config.isAppKeyValid()) {
            console.error("AppKey not set!");
        }

        if (this._autoPageView) {
            this.trackPageView();
        }

        this._loop();
    };

    pushAction(actionName, ...args) {
        this._actionQueue.push([actionName, ...args]);
    }

    _loop() {

        this._processFunctionCall();

        const self = this;

        setTimeout(function () {
            self._loop();
        }, 1);
    };

    _processFunctionCall() { 
        const CIData = window.CIData || [];

        const actions = CIData.concat(this._actionQueue);

        CIData.splice(0, CIData.length);
        this._actionQueue.splice(0, this._actionQueue.length);

        while (actions.length > 0) {
            /**
             * @type {Array}
             */
            const item = actions.shift();
            const functionName = item[0];
            const args = item.slice(1);

            if (this[functionName] !== undefined) {
                // console.log("call " + functionName + " with args:" + args.join(","));
                const f = this[functionName];
                f.apply(this, args);
            }
            else {
                console.warn("function not exist:" + functionName + " for" + item.join(","));
            }
        }
    };

    setAppKey(appKey) {
        Config.setAppKey(appKey);
    };

    /**
     *
     * @param autoPageView bool
     */
    setAutoPageView(autoPageView) {
        this._autoPageView = autoPageView;
    };

    /**
     * @param url
     * @param referrer
     */
    trackPageView(url, referrer) {
        url = url || window.location.href;

        referrer = referrer || document.referrer;

        Server.sendPV(url, referrer);
    };

    trackEvent(category, action, label, value) {
        Server.sendAction(category, action, label, value);
    };

    setLogHandler(logHandler) {
        Log.getInstance().setLogHandler(logHandler);
    };

    actionTimeStart(action, extdata) {
        Server.actionTimeStart("custom", action, extdata)
    };

    actionTimeEnd(action) {
        Server.actionTimeEnd(action)
    };

    setUserId(userId) {
        LocalData.setItem("user_id", userId);
    };

    setChannel(channel) {
        LocalData.setItem("channel", channel);
    };

    setUserProperties(properties) {
        UserProperty.getInstance().setProperties(properties)
    };


    setVersion(version) {
        LocalData.setItem("version", version);
    };

    static getInstance() {
        if (g_instance === null) {
            g_instance = new App();
        }
        return g_instance;
    };
}

