import * as LocalData from "./LocalData";
import UserProperty from "./UserProperty";
import * as Config from "./Config";
import * as Net from "./Net";
import Log from "./Log";
import * as EventSender from "./EventSender";
import ActionTime from "./ActionTime";
import * as EventId from "./EventId";


const DEVICE_ID_KEY = "cidata_uid";
const USER_ID_KEY = "user_id";

function processDeviceId(result) {
    try {
        const result_obj = JSON.parse(result);
        const uid = result_obj["uid"];
        if (uid && uid !== "") {
            LocalData.setItem(DEVICE_ID_KEY, uid);
        }
    }
    catch (e) {

    }
}

function appendDeviceId(data) {
    let uid = LocalData.getItem(DEVICE_ID_KEY);
    if (!uid || uid === "") {
        function guid() {
            function s4() {
                return Math.floor((1 + Math.random()) * 0x10000)
                    .toString(16)
                    .substring(1);
            }

            return s4() + s4() + '-' + s4() + '-' + s4() + '-' +
                s4() + '-' + s4() + s4() + s4();
        }

        uid = guid();
    }

    data["uid"] = uid;
    data["device_id"] = uid;
}

function appendVersionInfo(data) {
    data["version"] = Config.getVersion();
}

function appendInfo(data) {
    data["id"] = EventId.next();
    data["time"] = new Date().getTime();
}

function appendUserInfo(data) {
    data["user_properties"] = UserProperty.getInstance().getProperties();
    let channel = LocalData.getItem("channel");
    if (channel === null) {
        channel = "";
    }
    let userId = LocalData.getItem(USER_ID_KEY);
    if (userId === null) {
        userId = "";
    }

    let version = LocalData.getItem("version");
    if (version === null) {
        version = "";
    }

    data["channel"] = channel;
    data["user_id"] = userId;
    data["version"] = version;
}

export function sendPV(url, referrer) {
    const data = {appkey: Config.getAppKey(), url: url, referrer: referrer};

    appendDeviceId(data);

    appendVersionInfo(data);

    appendInfo(data);

    appendUserInfo(data);

    Net.send(Config.getServer() + "/api/web/pv", data, function (result) {
        processDeviceId(result);
        Log.getInstance().add(result);
    }, null);
}

export function sendAction(category, action, label, value) {
    const data = {appkey: Config.getAppKey(), category: category, action: action, label: label, value: value};

    appendDeviceId(data);

    appendVersionInfo(data);

    appendInfo(data);

    appendUserInfo(data);

    EventSender.send(data);
}

export function actionTimeStart(type, action, extdata) {
    const data = {app_id: Config.getAppKey(), type: type, event_id: action, params: extdata};

    appendDeviceId(data);

    appendVersionInfo(data);

    appendInfo(data);

    appendUserInfo(data);

    data["platform"] = "web";

    const actionTime = ActionTime.getInstance(Config.getWSUrl());

    actionTime.start(data)
}

export function actionTimeEnd(action) {
    const data = {event_id: action};

    const actionTime = ActionTime.getInstance(Config.getWSUrl());

    actionTime.end(data)
}

